#version 3.7;

#ifndef( Nightime )
	#declare Nightime = false;
#end

#declare Rad = true;
#declare FloreActive = false;
#declare PlacePumpkins = true;
#if( Nightime )
	#declare LightInsidePumpkin = true;
	#declare MediaPentagram = true;
#else
	#declare MediaPentagram = false;
	#declare LightInsidePumpkin = false;
#end
#declare PlaceMushrooms = true;
#declare PlaceLeeks = true;
#declare PlaceWell = true;
#declare LampsLightOn = false;

global_settings{ assumed_gamma 1.0 
	#if( Rad )
		#default{ finish{ ambient 0 } }
	
		#local P_start = 64 / image_width;
		#local P_end_final = 4 / image_width;

		radiosity{
			pretrace_start P_start
			pretrace_end   P_end_final
			error_bound 0.2
			minimum_reuse 0.015
			nearest_count 9
			count 800
			recursion_limit 1
			always_sample off
			gray_threshold 0.6
			brightness 1
			adc_bailout 0.01/2
			normal on
			media on
		}
	#end
}

background{ color srgb 0.05 }

camera{
	location< -24, 6, -24>
	look_at <6, 5, 0>
}

light_source{ < -2, 4, -3 > * 1000000 color srgb < 0.5, 0.5, 0.6> * 0.75 }

#include "includes/customColors.inc"
#include "includes/customTextures.inc"
#include "includes/macroTimeSetter.inc"
#include "includes/outdoorsLamp.inc"
#include "includes/placeStones.inc"
#include "includes/pot.inc"
#include "includes/pumpkin.inc"
#include "includes/ground.inc"
#include "includes/woodenHouse.inc"
#include "includes/leeks.inc"
#include "includes/well.inc"

#if( FloreActive )
	#include "includes/tilleul.inc"
	#include "includes/bush.inc"
	#include "includes/placeFlore.inc"
#end

#if( MediaPentagram & Nightime )
	#include "includes/pentagramObject.inc"
#end

#if( PlaceMushrooms )
	#include "includes/mushroom1.inc"
#end

//The sky
#if( Nightime )
	#declare SkyPigment = pigment{ gradient y color_map{
													[0.0 color srgb 0 ]
													[0.75 color srgb <0.075, 0.075, 0.15> ]
													}
							}
#else
	#declare SkyPigment = pigment{ gradient y color_map{
													[0.0 color srgb <0.8, 0.8, 0.9 > ] //<0.8, 0.8, 0.9>
													[0.3 color srgb < 143, 195, 225> / 255 ] //< 143, 195, 225> / 255
													}
							}
#end
						
sky_sphere{ pigment{SkyPigment} }
						
sphere{0, 1 
		pigment{ SkyPigment }
		#if( Nightime )
			finish{ diffuse 0 emission <1, 1, 0.95> * 0.3 }
		#else
			finish{ diffuse 0 emission 3 } 
		#end
		scale 4000000 
		hollow on 
		no_shadow
		no_image
		no_reflection
}

object{ WholeGround }

#declare PositionsNum = 2;
#local BrownLampPositions = array[PositionsNum] { <-2*4, 0.5, 2*4>, <-2*4, 0.5, -5.5 * 4> } 
#local BrownLampScale = 1 / 58;

#for( It, 0, PositionsNum - 1, 1 )
	PlaceLampWithPosDetection( WholeGround, BrownLampPositions[It].x, BrownLampPositions[It].z, BrownLampScale )
	#if( LampsLightOn )
		PlaceLight( BrownLampPositions[It], BrownLampScale, color srgbft < 0.4, 0.4, 0.2, 0.5, 0.5> )
	#end
#end

object{ CompleteHouse translate 40 * z }

PlaceStones(<-2.5,0,-6*4>, <3.4,0,20*4>, 3, 2.8)

object{ PotObject translate (-min_extent(PotObject).y)*y scale 1.8 translate < 7, 0.375, -15.25 > }

#if( PlacePumpkins )
	MakePumpkin(< 2*4, 0.35, 8 >, < 0, 40, 0>, 1, false, PumpkinsColors[2], true, LightInsidePumpkin ) //Center orange pumpkin
	MakePumpkin(< 2*4, 0.35, -19>, < 0, 65, 0>, 1, false, PumpkinsColors[2], false, LightInsidePumpkin ) //Right orange pumpkin
	MakePumpkin(< -14, 0.35, 0>, < 0, 20, 0 >, 1, false, PumpkinsColors[3], false, LightInsidePumpkin ) //Left yellow pumpkin
#end

#if( FloreActive )
	PlaceBushes()
	PlaceTilleuls( 14 )
#end

#if( MediaPentagram )
	
	#local Norm = < 0, 0, 0>;
	#local StartPos = < -12, 100, -11>;
	#local FinalPos = trace( WholeGround, StartPos, -y, Norm );
	#if( Norm.x != 0 | Norm.y != 0 | Norm.z != 0 )
		object{ Pentagram scale PentagramScaleFactor translate FinalPos }
	#end
	
	//Placing candles
	#local CandleNorm = < 0, 0, 0>;
	#for( ItCandle, 0 PointsNumber - 1 )
		#local FinalCandlePos = trace( WholeGround, StartPos + CandlesCoordinates[ItCandle], -y, CandleNorm );
		#if( CandleNorm.x != 0 | CandleNorm.y != 0 | CandleNorm.z != 0 )
			MakeCandle( FinalCandlePos, 0, 0.35 + rand(CandleHighnessSeed) * 0.1, 
						< rand(CandleFireSeed) - 0.5, 0.25, rand(CandleFireSeed) - 0.5> )
		#end
	#end
#end

#if( PlaceMushrooms )
	#declare MushroomsScale = 0.35;
	MakeCustomMushroom( 5, 79230, 0.2, < 5, 0, 4 >, < 6.8, 0.25, -18.4 >, MushroomsScale )
	MakeCustomMushroom( 3, 1789, 0.3, < -5, 0, -4 >, < 7.5, 0.25, -17.6 >, MushroomsScale )
	MakeCustomMushroom( 4, 79, 0.25, < 0, 0, 0 >, < 7, 0.25, -21 >, MushroomsScale )
#end

#if( PlaceLeeks )
	PlaceLeeksRow( 12, 0.52, < 7.5, 0, 2 >, < 2, 0, 0 >, 1, WholeGround ) 
	PlaceLeeksRow( 12, 0.54, < 7.4, 0, -1 >, < 2, 0, 0 >, 2, WholeGround ) 
	PlaceLeeksRow( 12, 0.53, < 7.25, 0, -4 >, < 2, 0, 0 >, 3, WholeGround ) 
#end

#if( PlaceWell )
	object{ CompleteWell scale 3 translate < 5.4 + 4, 0.33448477, -9 > }
#end