
#declare LeekGreen = color srgb < 128, 178, 69 > / 255;
#declare LeekWhite = color srgb 1;
#declare LeekYellow = color srgb < 220, 240, 144 > / 255;
#declare LeekDarkGreen = color srgb < 89, 112, 99 > / 255;

#declare StalkGradient =
pigment{ gradient y color_map{ 	
								[ 0 LeekWhite ]
								[ 0.15 LeekWhite ]
								[ 0.25 LeekYellow ]
								[ 0.35 LeekYellow ]
								[ 0.55 LeekGreen ]
								[ 0.75 LeekGreen ]
								[ 1 LeekDarkGreen ] 
							} 
		scale < 1, 2.4 + 0.2, 1 > 
}

#declare LeekPigment = 
pigment{ cylindrical pigment_map{ 
								[ 0 LeekDarkGreen ]
								[ 0.4 LeekDarkGreen ]
								[ 0.5 StalkGradient ]
								[ 1 StalkGradient ]
							  }
	   } 

#declare LeekFinish = finish{ specular 0.1 roughness 0.001 }

#declare LeekTexture = texture{ pigment{ LeekPigment } finish{ LeekFinish } }

#declare BrancheRadius = 0.2;

#declare LeekTrunk =
cylinder{
	< 0, 0, 0>, < 0, 1, 0>, BrancheRadius 
	texture{ LeekTexture } 
}

#macro MakeSphereSweep1( AddLastSphere, TranslateVector, RadiusMulti )
	sphere_sweep{
		linear_spline
		
		#if( AddLastSphere )
			10,
		#else
			9,
		#end
		< 0, 0, 0 >, BrancheRadius * RadiusMulti,
		< 0, 1, 0.15 >, BrancheRadius * RadiusMulti,
		< 0, 1.5, 0.4 >, BrancheRadius * RadiusMulti,
		< 0, 2, 0.7 >, BrancheRadius * RadiusMulti,
		< 0, 2.25, 1 >, BrancheRadius * RadiusMulti,
		< 0, 2.4, 1.3 >, BrancheRadius * RadiusMulti,
		< 0, 2.3, 1.6 >, BrancheRadius * RadiusMulti,
		< 0, 2, 1.95 >, BrancheRadius * RadiusMulti,
		< 0, 1.5, 2 >, BrancheRadius * RadiusMulti 
		#if( AddLastSphere 
		)
			< 0, 1.4, 2 >, BrancheRadius * 2 * RadiusMulti
		#end	
		texture{ LeekTexture } 
		
		translate TranslateVector
	}
#end

#macro MakeSphereSweep2( AddLastSphere, TranslateVector, RadiusMulti )
	sphere_sweep{
		linear_spline
		#if( AddLastSphere )
			8,
		#else
			7,
		#end
		< 0, 0, 0 >, BrancheRadius * RadiusMulti,
		< 0, 1, 0.175 >, BrancheRadius * RadiusMulti,
		< 0, 1.5, 0.35 >, BrancheRadius * RadiusMulti,
		< 0, 1.75, 0.45 >, BrancheRadius * RadiusMulti,
		< 0, 2, 0.525 >, BrancheRadius * RadiusMulti,
		< 0, 2.25, 0.575 >, BrancheRadius * RadiusMulti,
		< 0, 2.35, 0.65 >, BrancheRadius * RadiusMulti
		#if( AddLastSphere
		)
			< 0, 2.4, 0.75 >, BrancheRadius * 2 * RadiusMulti
		#end	
		texture{ LeekTexture }  
	
		translate TranslateVector
	}
#end

#declare LeekBranche1 =
difference{
	MakeSphereSweep1(false, 0, 1)
	MakeSphereSweep1(true, BrancheRadius / 3 * y, 1)
}

#declare LeekBranche2 =
difference{
	MakeSphereSweep2(false, 0, 1)
	MakeSphereSweep2(true, BrancheRadius / 3 * y, 1)
}

#declare Leek1 =
union{
	object{ LeekTrunk }
	object{ LeekBranche1 translate 0.25 * y }
	object{ LeekBranche1 rotate 120 * y translate 0.25 * y }
	object{ LeekBranche1 rotate 240 * y translate 0.25 * y }
}

#declare Leek2 =
union{
	object{ LeekTrunk scale < 1, 0.5, 1 > }
	object{ LeekBranche2 translate 0.25 * y }
	object{ LeekBranche2 rotate 120 * y translate 0.25 * y }
	object{ LeekBranche2 rotate 240 * y translate 0.25 * y }
}

#macro PlaceLeeksRow(LeeksNumber, LeekScale, StartingPos, Spacing, SeedNumber, GroundObject)
	#local LeekTypeSeed = seed( SeedNumber );
	#local Position = StartingPos;
	#local Norm = < 0, 0, 0 >;
	#for( Num, 0, LeeksNumber - 1 )
		#local LeekType = rand( LeekTypeSeed );
		#local NewIntersection = trace( GroundObject, < Position.x, 1000000, Position.z >, -y, Norm );
			
		#if( Norm.x != 0 | Norm.y != 0 | Norm.z != 0 )
			#if( LeekType < 0.5 )
				object{ Leek1 scale LeekScale translate < Position.x, NewIntersection.y, Position.z > }
			#else
				object{ Leek2 scale LeekScale translate < Position.x, NewIntersection.y, Position.z > }
			#end
			#local Position = Position + Spacing;
		#end
	#end
#end