//An outdoor lamp

#declare FootSplinePtNum = 50;
#declare BaseFootSide = 40;
#declare BasePyramidSide = BaseFootSide - 10;
#declare BasePyramidHighness = 274;
#declare FootLength = 47;
#declare FootTorusMinRadius = 7;

#declare LampStructColor = color srgb < 66, 49, 42 > / 255;
#declare LampStructFinish = finish{ specular 0.05 roughness 0.25 metallic }
#declare LampStructTexture = texture{ pigment{ LampStructColor } finish{ LampStructFinish } }                              

#declare FirstRingRadius = 25;

#declare LampFoot =
difference{
	cylinder{ < 0, 0, 0 >, < 0, 7, 0 >, FootLength - vrotate( <0, -7,0 >, < 0, 0, 90 / ( FootSplinePtNum - 1 ) * 0 >).x }
	torus{ FootLength, FootTorusMinRadius translate 7 * y } 
	texture{ LampStructTexture }
}
    
#declare BasePyramid =
prism{ conic_sweep linear_spline 1, 0.5, 5 < -BasePyramidSide / BasePyramidHighness, 0>, 
                                           < 0, BasePyramidSide / BasePyramidHighness >, 
                                           < BasePyramidSide / BasePyramidHighness, 0 >, 
                                           < 0, -BasePyramidSide / BasePyramidHighness>,
                                           < -BasePyramidSide / BasePyramidHighness, 0> 
    rotate 180*x
    rotate 45*y
    translate y                                           
    scale BasePyramidHighness
    texture{ LampStructTexture }
}

#declare BumpPyramid =
prism{ conic_sweep linear_spline 0.95, 0.55, 5 < -BasePyramidSide / BasePyramidHighness, 0>, 
                                           < 0, BasePyramidSide / BasePyramidHighness >, 
                                           < BasePyramidSide / BasePyramidHighness, 0 >, 
                                           < 0, -BasePyramidSide / BasePyramidHighness>,
                                           < -BasePyramidSide / BasePyramidHighness, 0> 
    rotate 180*x
    rotate 45*y
    translate y                                           
    scale < BasePyramidHighness * 1.15, BasePyramidHighness, BasePyramidHighness * 0.7 >
    texture{ LampStructTexture }
}

//Rings
#declare FR_TorusSmallRadius = 7;
#declare FR_TorusBigRadius = 18;
#declare FR_SphereRadius = FR_TorusSmallRadius + FR_TorusBigRadius;
#declare FR_BottomSphere = sphere{ 0, 1 scale < FR_SphereRadius, 
												(FR_SphereRadius) / 1.5 , 
												FR_SphereRadius > }
#declare FR_MiddleTorus = torus{ FR_TorusBigRadius, FR_TorusSmallRadius sturm }

#declare FirstRingYScale = 0.8;
#declare SecondRingYScale = 1.3;
#declare FirstRingYPos = BasePyramidHighness * 0.5 + 14;
#declare SecondRingYPos = FirstRingYPos + (FR_TorusSmallRadius * 2 + FR_SphereRadius / 1.5 ) * FirstRingYScale;

#declare SmallRingYPos = SecondRingYPos + SecondRingYScale * (FR_TorusSmallRadius * 2 + FR_SphereRadius / 1.5 );
#declare LongStalkHighness = 414;

#declare CompleteFirstRing = 
union{
	difference{
		object{ FR_BottomSphere }
		box{ < -FR_SphereRadius, 0, -FR_SphereRadius >, < FR_SphereRadius, FR_SphereRadius, FR_SphereRadius > }
		translate FR_SphereRadius / 1.5 * y 
	}
	object{ FR_MiddleTorus translate FR_SphereRadius / 1.5 * y }
	difference{
		cylinder{ <0,0,0>, <0, -FR_TorusSmallRadius,0>, FR_TorusBigRadius }
		object{ FR_MiddleTorus }
		translate (FR_TorusSmallRadius * 2 + FR_SphereRadius / 1.5) * y
	}
	texture{ LampStructTexture }
}

#declare LampCageScale = 48 / 2;
#declare BrancheRadius = 3.5 / LampCageScale;
#declare BrancheHorizStart = 0.25;
#declare LampBrancheYPos = SmallRingYPos + LongStalkHighness;
#declare LampCageYPos = SmallRingYPos + LongStalkHighness + LampCageScale;

#declare LampCageBranche =
sphere_sweep{
	cubic_spline
	7,
	< -BrancheHorizStart, -1, - BrancheHorizStart>, BrancheRadius,
	< -BrancheHorizStart, -0.5, - BrancheHorizStart>, BrancheRadius,
	< -0.4, 0, -0.4>, BrancheRadius,
	< -0.6, 0.25, -0.6>, BrancheRadius,
	< -0.8, 0.35, -0.8>, BrancheRadius,
	< -1, 0.5, -1>, BrancheRadius,
	< -1.125, 0.5, -1.125>, BrancheRadius
	//tolerance 1.0e-4
	translate 0.5 * y
	scale LampCageScale
	texture{ LampStructTexture }
}

#declare LampCageEdgeFactor = 1/32;
#declare CageScaleMin = 0.9;
#declare CageScaleMax = 1.5;

#declare LampCage = 
difference{
	prism{ conic_sweep linear_spline 1, 2/3, 5, < -0.5, 0.5 >, < 0.5, 0.5 >, < 0.5, -0.5 >, < -0.5, -0.5 >, < -0.5, 0.5 >
		translate -2 / 3 * y
	}
	prism{ conic_sweep linear_spline 1 - LampCageEdgeFactor, 2/3 + LampCageEdgeFactor, 5
		, < -0.5, 0.5 >, < 0.5, 0.5 >, < 0.5, -0.5 >, < -0.5, -0.5 >, < -0.5, 0.5 >
		translate -2 / 3 * y
		scale < CageScaleMin, 1, CageScaleMax >
	}
	prism{ conic_sweep linear_spline 1 - LampCageEdgeFactor, 2/3 + LampCageEdgeFactor, 5
		, < -0.5, 0.5 >, < 0.5, 0.5 >, < 0.5, -0.5 >, < -0.5, -0.5 >, < -0.5, 0.5 >
		translate -2 / 3 * y
		scale < CageScaleMax, 1, CageScaleMin >
	}
	scale < 3, 9, 3>
	scale LampCageScale
	translate LampCageYPos * y
	texture{ LampStructTexture }
}

#declare FirstHatYPos = LampCageYPos + 3 * LampCageScale;
#declare FirstHatSide = 3;
#declare FirstHatYScale = 1.2;

#declare FirstLampHat =
prism{ conic_sweep linear_spline 1, 0.75,5,
		<-FirstHatSide, FirstHatSide>,<FirstHatSide, FirstHatSide>,
		<FirstHatSide,-FirstHatSide>,<-FirstHatSide,-FirstHatSide>,<-FirstHatSide,FirstHatSide>
	rotate 180*x
	translate y
	scale LampCageScale * < 0.6, FirstHatYScale, 0.6 >
	translate FirstHatYPos * y
	texture{ LampStructTexture }
}

#declare SecondHatYPos = FirstHatYPos + 0.25 * FirstHatYScale;
#declare SecondHatSide = 1/3;
#declare SecondHatScale = <4,5.5,4>;

#declare SecondLampHat =
prism{ conic_sweep linear_spline 1, 7/9, 5, 
		<-SecondHatSide,SecondHatSide>,<SecondHatSide,SecondHatSide>,
		<SecondHatSide,-SecondHatSide>,<-SecondHatSide,-SecondHatSide>,<-SecondHatSide,SecondHatSide>
	rotate 180*x
	translate y
	scale LampCageScale * SecondHatScale
	translate SecondHatYPos * y
	texture{ LampStructTexture }
}

#declare CompleteLamp =
    union{
        object{ LampFoot }
        object{ LampFoot scale <1 / FootLength * BaseFootSide, 1, 1 / FootLength * BaseFootSide> translate 7*y }
        object{ BasePyramid translate 14*y }
        object{ BumpPyramid translate 14*y }
        object{ BumpPyramid translate 14*y rotate 90*y}
        object{ CompleteFirstRing scale <1,FirstRingYScale,1> translate FirstRingYPos * y }
        object{ CompleteFirstRing scale <0.8, SecondRingYScale, 0.8 > translate SecondRingYPos * y }
		cylinder{ 	< 0, FirstRingYPos, 0 >, 
					< 0, SecondRingYPos + SecondRingYScale * (FR_TorusSmallRadius * 2 + FR_SphereRadius / 1.5 ) * 0.5, 0 >,
					FR_TorusBigRadius - FR_TorusSmallRadius
					texture{ LampStructTexture }
		}
		torus{ FR_TorusSmallRadius, FR_TorusSmallRadius / 2 
				translate SmallRingYPos * y
				texture{ LampStructTexture } }
		cylinder{ < 0, SmallRingYPos, 0  >,< 0, SmallRingYPos + LongStalkHighness, 0>, FR_TorusSmallRadius 
				texture{ LampStructTexture } }
		torus{ FR_TorusSmallRadius + FR_TorusSmallRadius / 2, FR_TorusSmallRadius 
				translate (SmallRingYPos + LongStalkHighness) * y 
				texture{ LampStructTexture } }
		#for( Rot, 0, 3 )
			object{ LampCageBranche rotate Rot * 90 * y translate LampBrancheYPos * y }
		#end
		object{ LampCage }
		object{ FirstLampHat }
		object{ SecondLampHat }
    }

#macro PlaceLight( LampPosParam, LampScaleParam )
	#local NewPosition = LampPosParam + < 0, LampCageYPos * LampScaleParam * 1.05, 0>;
	sphere{ 0, 1 hollow no_shadow pigment{ color srgbft <0, 0, 0, 0, 1> }
		interior{
			media{
				emission color srgb 1 samples 1, 30 intervals 10
				density{ spherical density_map{
											[0 color srgb 0 ]
											[0.2 color srgb <1, 1, 0> ]
											[0.5 color srgb <1, 0, 0> ]
											[1 color srgb <1, 0, 0> ]
									}
					warp{ turbulence < 0.25, 0.5, 0.25 > }
				}
			}
		}
		scale 0.25
		translate NewPosition
	}
	light_source{ NewPosition color srgb < 0.5, 0.45, 0.4>*0.8 fade_distance 40 fade_power 1 }
#end

#macro PlaceLampWithPosDetection( SurfaceObject, LampXPos, LampZPos, LampScale )
	#local Norm = < 0, 0, 0 >;
	#local LampPos = trace( SurfaceObject, < LampXPos, 1000000, LampZPos >, -y , Norm );
	#if( Norm.x != 0 | Norm.y != 0 | Norm.z != 0 )
		object{ CompleteLamp scale LampScale translate LampPos }
	#else
		#debug concat( "Bad Coord value to set OutdoorLamp in scene: x = ", str(LampXPos, 4, 4), " z = ", str(LampZpos, 4, 4) , " \n")
	#end
#end

//Display the size of the lamp
#declare LampSize = max_extent( CompleteLamp ) - min_extent( CompleteLamp );
#debug concat( "Lamp size: [", vstr( 3, LampSize, " ,", 0, 5 ) , " ]\n")
