
#include "woods.inc"

#macro AffineSet( PointA, PointB, CurrentX )
    #if( PointA.x = PointB.x 
    )
        #error "Can't render, because both x coordinates have equal values"
    #end
    
	#local DirCoef = (PointB.y - PointA.y) / ( PointB.x - PointA.x );
	#local OriginOffset = PointA.y - (DirCoef * PointA.x);
	(DirCoef * CurrentX) + OriginOffset;
#end

//Textures, pigments, normals and finishes                                                         
#declare StalksTexture = texture{ T_Wood16 }
#declare RoofTexture = texture{ T_Wood12 }

#declare StoneColor = color srgb 1;
#declare MortarColor = color srgb 0.65;

#declare WellStonesPig = pigment{ wood color_map{ 
											[ 0.705 StoneColor ]//0.711
											[ 0.705 MortarColor]
											[ 0.9 MortarColor ]
											[ 0.9 StoneColor ]
											[ 1.0 StoneColor ] } 
							rotate 90 * x
							scale < 2.24, 1, 2.24 >
						}
						
#declare StoneNormal = normal{ wrinkles 1 scale 0.0025 }

#declare WellBaseTexture = texture{ 
								pigment{ WellStonesPig } 
								normal{ StoneNormal } 
							}
							
#declare WellCoverTexture = texture{
								pigment{ StoneColor }
								normal{ StoneNormal }
							}

#declare RopeBarTexture = texture{ pigment{color srgb 0.5 } finish{specular 0.2 roughness 0.01} }

#declare RopeColor = color srgb < 190, 183, 181 > / 255;
#declare RopeTexture = texture{ pigment{ RopeColor } finish{ specular 0.001 roughness 0.001 } }

#declare BucketMetalTexture = texture{ pigment{ color srgb 0.5 } finish{ specular 0.1 roughness 0.1 } }

//End textures enums

#declare Stalk1_Z_Base = 1;
#declare Stalk1_Highness = 0.4;
#declare Stalk1_Large = 0.75;
#declare Stalk1_Width = 0.1;
      
#declare Stalk2_Width = 0.125;

#declare Stalk3_Width = 0.1;    
#declare Stalk3_Z_BottomStart = 0.5;
#declare Stalk3_Large = 0.45;

#declare Stalk4_Width = 0.075;
#declare Stalk4_Large = Stalk1_Large - 0.15;
               
#declare LinearStalk1_PointA = < 0, Stalk1_Z_Base + Stalk1_Highness >;
#declare LinearStalk1_PointB = < -Stalk1_Large, Stalk1_Z_Base >;

#declare Stalk5_X_Decal = 0.08;
#declare Stalk5_Width = 0.07;
#declare Stalk5_MajorZ = AffineSet( LinearStalk1_PointA, LinearStalk1_PointB, -Stalk5_X_Decal ) 
#declare Stalk5_MinorZ = AffineSet( LinearStalk1_PointA, LinearStalk1_PointB, -Stalk5_X_Decal - Stalk5_Width ) 

#declare WellRoofWidth = 2.1;
#declare WellRoofThickness = 0.025;
               
#declare Stalk1_Obj =
prism{
    linear_sweep linear_spline
    Stalk1_Width / 2, -Stalk1_Width / 2, 7
    //Start from top coord
    < 0, Stalk1_Z_Base + Stalk1_Highness + Stalk1_Width >, 
    < -Stalk1_Large, Stalk1_Z_Base + Stalk1_Width >, 
    < -Stalk1_Large, Stalk1_Z_Base >,
    < 0, Stalk1_Z_Base + Stalk1_Highness >, 
    < Stalk1_Large, Stalk1_Z_Base >,
    < Stalk1_Large, Stalk1_Z_Base + Stalk1_Width >,
    < 0, Stalk1_Z_Base + Stalk1_Highness + Stalk1_Width >    
    
    texture{ StalksTexture }
}

#declare Stalk2_Obj =
box{
    < -Stalk2_Width / 2, -Stalk2_Width / 2, 0 >, < Stalk2_Width / 2,Stalk2_Width / 2, Stalk1_Z_Base >
    texture{ StalksTexture }
}

#declare Stalk3_Obj =
prism{
    linear_sweep linear_spline
    -Stalk3_Width / 2, Stalk3_Width / 2, 5
    < Stalk2_Width / 2, Stalk3_Z_BottomStart >,
    < Stalk2_Width / 2, Stalk3_Z_BottomStart + Stalk3_Width >,
    < Stalk3_Large, Stalk1_Z_Base >,
    < Stalk3_Large + Stalk3_Width, Stalk1_Z_Base >,
    < Stalk2_Width / 2, Stalk3_Z_BottomStart >
    
    texture{ StalksTexture }
}   

#declare Stalk4_Obj =
box{
    < -Stalk4_Large, -Stalk4_Width, Stalk1_Z_Base>,
    < Stalk4_Large, Stalk4_Width, Stalk1_Z_Base + Stalk4_Width > 
    texture{ StalksTexture }
}
   
#declare Stalk5_Obj =
prism{
    linear_sweep linear_spline
    -Stalk5_Width / 2, Stalk5_Width / 2, 5
    < -Stalk5_X_Decal, Stalk1_Z_Base + Stalk4_Width >,
    < -Stalk5_X_Decal - Stalk5_Width, Stalk1_Z_Base + Stalk4_Width >,
    < -Stalk5_X_Decal - Stalk5_Width, Stalk5_MinorZ >,
    < -Stalk5_X_Decal, Stalk5_MajorZ >,
    < -Stalk5_X_Decal, Stalk1_Z_Base + Stalk4_Width >     
    texture{ StalksTexture }
}
   
#declare WellSide =
union{
    object{ Stalk1_Obj }
    object{ Stalk2_Obj }
    object{ Stalk3_Obj } 
    object{ Stalk3_Obj rotate 180 * z }  
    object{ Stalk4_Obj }    
    object{ Stalk5_Obj }
    object{ Stalk5_Obj rotate 180 * z }
}        

#declare WellRoofObj =
prism{
	linear_sweep linear_spline
	-WellRoofWidth / 2, WellRoofWidth / 2, 7
	< 0, Stalk1_Z_Base + Stalk1_Highness + Stalk1_Width >, 
	< Stalk1_Large, Stalk1_Z_Base + Stalk1_Width >, 
	< Stalk1_Large, Stalk1_Z_Base + Stalk1_Width + WellRoofThickness >, 
	< 0, Stalk1_Z_Base + Stalk1_Highness + Stalk1_Width + WellRoofThickness >,
	< -Stalk1_Large, Stalk1_Z_Base + Stalk1_Width + WellRoofThickness >, 
	< -Stalk1_Large, Stalk1_Z_Base + Stalk1_Width >, 
	< 0, Stalk1_Z_Base + Stalk1_Highness + Stalk1_Width >
	
	texture{ RoofTexture }
}

#declare WellBaseRadius = 1;
#declare WellBaseDiffRadius = 0.8;
#declare WellBaseCoverRadius = 1.025;
#declare WellBaseThickness = 0.2; 
#declare WellHighness = 0.65;
#declare WellBaseCoverHighness = 0.05;

#declare CylinderFunc = function{ x * x + z * z - WellBaseRadius * WellBaseRadius }
#declare CylinderDiffFunc = function{ x * x + z * z - WellBaseDiffRadius * WellBaseDiffRadius }
#declare CylinderCoverFunc = function{ x * x + z * z - WellBaseCoverRadius * WellBaseCoverRadius }

#declare CracklePigFunc = function{ pigment{ crackle pigment_map{
															[ 0.06 color srgb 0.1 ]
															[ 0.09 color srgb 0.2 ]
													}
										scale < 0.02, 0.075, 1 > * 1.5
										//scale < 0.02, 0.075, 1 > old version of scale
										warp{ cylindrical }
									}
							}
							
#declare WrinklesPig = pigment{ wrinkles }

#declare WrinklesPigFunc = function{ pigment{ WrinklesPig } }

#declare WellBaseReal =
isosurface{
	function{ CylinderFunc(x,y,z) 
				- CracklePigFunc(x,y,z).gray 
	}
	threshold 0
	accuracy 0.01
	max_gradient 46.2
	contained_by{ box{  < -WellBaseRadius * 1.0086, 0 , -WellBaseRadius * 1.0086 >, 
                        < WellBaseRadius * 1.0086, WellHighness, WellBaseRadius * 1.0065 > } 
	}
    max_trace 2
    texture{ WellBaseTexture }
}

#declare WellBaseDiff =
isosurface{
	function{  CylinderDiffFunc(x,y,z) 
				+ CracklePigFunc(x,y,z).gray 
	}
	threshold 0
	accuracy 0.01
	max_gradient 46.2
	contained_by{ box{  < -WellBaseDiffRadius * 0.999, -1, -WellBaseDiffRadius * 0.999 >, 
                        < WellBaseDiffRadius * 0.999, WellHighness + 1, WellBaseDiffRadius * 0.999 > } 
	}
	max_trace 2
	texture{ WellBaseTexture }
}

#declare WellBaseCover =
isosurface{
	function{ CylinderCoverFunc(x,y,z)
			- WrinklesPigFunc(x,y,z).gray * 0.1
	}
	threshold 0
	accuracy 0.01
	max_gradient 3.47
	contained_by{ box{  < -WellBaseCoverRadius * 1.035, 0, -WellBaseCoverRadius * 1.035 >, 
                        < WellBaseCoverRadius * 1.035, WellBaseCoverHighness, WellBaseCoverRadius * 1.035 > }
	}
	max_trace 2
	texture{ WellCoverTexture }
}

#declare WellBase =
difference{
	object{ WellBaseReal }
	object{ WellBaseDiff }
	texture{ WellBaseTexture }
}

#declare FinalWellBaseCover =
difference{
	object{ WellBaseCover }
	object{ WellBaseCover translate -WellBaseCoverHighness / 2 * y 
		scale < WellBaseDiffRadius / WellBaseCoverRadius, 2.1, WellBaseDiffRadius / WellBaseCoverRadius > }
	translate WellHighness * y
}

#declare BarRadius = 0.04;
#declare BarHalfSide = WellBaseRadius - WellBaseThickness / 2;
#declare BarYPosition = WellHighness * 2.25;

#declare WellRopeBar =
cylinder{ 	< 0, BarYPosition, -BarHalfSide >,
			< 0, BarYPosition, BarHalfSide > , BarRadius 
			texture{ RopeBarTexture }
}

#declare RoundNumber = 30;
#declare PerRoundSpheresNumber = 36;
#declare TotalSpheresNumber = RoundNumber * PerRoundSpheresNumber;
#declare RopeRadius = 0.01;
#declare RopeZStart = BarHalfSide * 8 / 10;


#declare Rope =
sphere_sweep{
	linear_spline
	TotalSpheresNumber,
	#for( It, 0, TotalSpheresNumber - 2 )
		#local SpherePos = vrotate( < BarRadius + RopeRadius, 0, RopeZStart - It * RopeRadius * 2 / PerRoundSpheresNumber >, 
									< 0, 0, It * 360 / PerRoundSpheresNumber - 180 >);
		SpherePos, RopeRadius,  
	#end
	#declare FinalRopeSpherePos = vrotate( < BarRadius + RopeRadius, 0, RopeZStart - TotalSpheresNumber * RopeRadius * 2 / PerRoundSpheresNumber >, 
									< 0, 0, TotalSpheresNumber * 360 / PerRoundSpheresNumber - 180 >);
	FinalRopeSpherePos, RopeRadius
	#debug concat("Last Sphere position: ", vstr( 3, FinalRopeSpherePos, ", ", 0, 8), "\n")
	translate BarYPosition * y
	texture{ RopeTexture }
}
//Last Sphere of rope coord is: < -0.06, 0, 0.08 >

#declare BucketMaxRadius = 0.2;
#declare BucketMinRadius = 0.16;
#declare BucketReservoirHighness = 0.3;
#declare AttacheMinRadius = 0.01;
#declare AttacheMaxRadius = 0.05;
#declare BucketTrunk = cone{ < 0, 0, 0 >, BucketMinRadius, < 0, BucketReservoirHighness, 0 >, BucketMaxRadius }
#declare BucketReservoir =
difference{
	object{ BucketTrunk }
	object{ BucketTrunk translate 0.01 * y scale < 0.96, 1, 0.96 >}
}
#declare BucketAttache = 
difference{ 
	torus{ AttacheMaxRadius, AttacheMinRadius rotate 90 * z}
	box{ < -1,-1,-1>,<1,0, 1 > }
}

#declare BucketAnse =
difference{
	torus{ BucketMaxRadius - AttacheMaxRadius, AttacheMinRadius rotate 90 * z }
	box{ < -1, -1, -1 >, < 1, 0, 1 > }
}

#declare CompleteBucket =
union{
	object{ BucketReservoir }
	object{ BucketAttache rotate 90 * y translate < 0, BucketReservoirHighness, -BucketMaxRadius > }
	object{ BucketAttache rotate 90 * y translate < 0, BucketReservoirHighness, BucketMaxRadius > }
	object{ BucketAttache rotate 180 * z translate < 0, BucketReservoirHighness + 2 * AttacheMaxRadius - AttacheMinRadius * 2, -BucketMaxRadius > }
	object{ BucketAttache rotate 180 * z translate < 0, BucketReservoirHighness + 2 * AttacheMaxRadius - AttacheMinRadius * 2, BucketMaxRadius > }
	object{ BucketAnse translate < 0, BucketReservoirHighness + AttacheMaxRadius - AttacheMinRadius * 2 + AttacheMaxRadius, 0 > }
	texture{ BucketMetalTexture }
}

#declare DirectBucketRope = 
union{
	cylinder{ FinalRopeSpherePos + < 0, BarYPosition, 0 >, 
			< FinalRopeSpherePos.x, 
			BucketReservoirHighness + AttacheMaxRadius - AttacheMinRadius * 2 + AttacheMaxRadius + BucketMaxRadius - AttacheMaxRadius + WellHighness, 
			FinalRopeSpherePos.z >,
			RopeRadius
	}
	sphere{ < FinalRopeSpherePos.x, 
			BucketReservoirHighness + AttacheMaxRadius - AttacheMinRadius * 2 + AttacheMaxRadius + BucketMaxRadius - AttacheMaxRadius + WellHighness, 
			FinalRopeSpherePos.z >,
			RopeRadius
	}
	texture{ RopeTexture }
}

#declare CompleteWell =
union{
	object{ WellBase }
	object{ FinalWellBaseCover }
	object{ WellRopeBar }
	object{ Rope }
	object{ DirectBucketRope }
	object{ CompleteBucket translate WellHighness * y translate < FinalRopeSpherePos.x, 0, FinalRopeSpherePos.z > }
	object{ WellRoofObj rotate -90 * x translate (WellHighness + WellBaseCoverHighness) * y }
    object{ WellSide rotate -90 * x translate < 0, WellHighness + WellBaseCoverHighness, -WellBaseRadius + WellBaseThickness / 2 > }
    object{ WellSide rotate -90 * x translate < 0, WellHighness + WellBaseCoverHighness,  WellBaseRadius - WellBaseThickness / 2 > }
    object{ Stalk1_Obj rotate -90 * x translate < 0, WellHighness + WellBaseCoverHighness, -(WellBaseRadius + WellBaseThickness / 2 ) / 2 > }
    object{ Stalk1_Obj rotate -90 * x translate < 0, WellHighness + WellBaseCoverHighness,  (WellBaseRadius + WellBaseThickness / 2 ) / 2 > }
    object{ Stalk1_Obj rotate -90 * x translate < 0, WellHighness + WellBaseCoverHighness,  0 > }
}