
#declare BlockInterval = 4;
#declare MinCorner = < 3 * BlockInterval, 0.25 * BlockInterval, -4 * BlockInterval >;
#declare MaxCorner = < 6 * BlockInterval, 7 / 4 * BlockInterval, 0 * BlockInterval >;
#declare WoodBoardWidth = 0.5;
#declare WoodBoardFactor = 0.2;
#declare WoodBoardSide = WoodBoardWidth * WoodBoardFactor;
#declare EndInterval = 1;
#declare LargeWallScale = (MaxCorner.z - MinCorner.z) / ( MaxCorner.x - MinCorner.x);

#declare HouseWoodTexture = texture{ pigment{ brown2_srgb } }

#declare DoorWidth = BlockInterval;

#declare DoorMinCorner = <MinCorner.x, MinCorner.y, ( MinCorner.z + MaxCorner.z ) / 2 - DoorWidth / 2 >;
#declare DoorMaxCorner = <MinCorner.x - WoodBoardSide, MaxCorner.y, ( MinCorner.z + MaxCorner.z ) / 2 + DoorWidth / 2 >;

//On x axis wood board
#declare XWoodBoard = prism{ linear_sweep linear_spline 0, -(MaxCorner.x - MinCorner.x), 5, 
								< 0, 0>, 
								< WoodBoardWidth, 0 >, 
								< WoodBoardWidth - WoodBoardSide, WoodBoardSide >,
								< WoodBoardSide, WoodBoardSide >,
								< 0, 0> 
							texture{ HouseWoodTexture }
						}

#declare ZWoodBoard = object{ XWoodBoard 
								translate -WoodBoardWidth * x 
								rotate -90 * y 
								scale <1, LargeWallScale ,1> 
								rotate 90 * x
						}

#declare HouseLeftWall =
union{
	#for( ItY, MinCorner.y, MaxCorner.y - WoodBoardWidth, WoodBoardWidth )
		object{ XWoodBoard rotate 90*z translate < MinCorner.x, ItY, MaxCorner.z> }
	#end
}

#declare HouseRightWall =
union{
	#for( ItY, MinCorner.y, MaxCorner.y - WoodBoardWidth, WoodBoardWidth )
		object{ XWoodBoard rotate 180 * y translate WoodBoardWidth * x rotate 90*z 
				translate < MinCorner.x, ItY, MinCorner.z> }
	#end
}

#declare HouseRearWall = 
union{
	#for( ItY, MinCorner.y, MaxCorner.y - WoodBoardWidth, WoodBoardWidth )
		object{ ZWoodBoard rotate 180 * y translate < MaxCorner.x, ItY, MinCorner.z > }
	#end
}

#declare HouseFrontWall = 
difference{
	union{
		#for( ItY, MinCorner.y, MaxCorner.y - WoodBoardWidth, WoodBoardWidth )
			object{ ZWoodBoard translate < MinCorner.x, ItY, MaxCorner.z > }
		#end
	}
	box{ < (MinCorner.x + MaxCorner.x) / 2, MinCorner.y, DoorMinCorner.z >,
		 < MinCorner.x - BlockInterval, MaxCorner.y, DoorMaxCorner.z > texture{ HouseWoodTexture } }
}

#declare HouseFondations = box{ MinCorner, MaxCorner - < 0, MaxCorner.y, 0> 
								texture{ pigment{ GrayPig } }
							}

#declare DoorWoodBoard = prism{ linear_sweep linear_spline MaxCorner.y, MinCorner.y, 5 
								< 0, 0>, 
								< 0, -WoodBoardWidth >, 
								< -WoodBoardSide, -WoodBoardWidth + WoodBoardSide >,
								< -WoodBoardSide, -WoodBoardSide >,
								< 0, 0> 
								texture{ HouseWoodTexture }
						 }


#declare DoorObject = 
union{
	#for( ItX, 0, BlockInterval / WoodBoardWidth )
		object{ DoorWoodBoard translate <DoorMaxCorner.x, 0, DoorMaxCorner.z> + ItX * WoodBoardWidth * -z }
	#end
	box{ 	< DoorMaxCorner.x - WoodBoardSide, (MinCorner.y + MaxCorner.y) / 2,DoorMinCorner.z >,
			< DoorMaxCorner.x - 2 * WoodBoardSide, (MinCorner.y + MaxCorner.y) / 2 + WoodBoardWidth, DoorMinCorner.z + 2 * WoodBoardWidth> texture{ HouseWoodTexture } }
}

#declare RoofObject =
prism{ linear_sweep linear_spline 0, MaxCorner.z - MinCorner.z, 4, 
		<0, 0>, < MaxCorner.x - MinCorner.x, 0>, < (MaxCorner.x - MinCorner.x ) / 2, (MaxCorner.x - MinCorner.x) / 4 >, <0, 0>
		rotate -90 * x translate < MinCorner.x, MaxCorner.y, MaxCorner.z >
		texture{ pigment{ GrayPig } }
}

#declare CompleteHouse =
union{
	object{ HouseLeftWall }
	object{ HouseRightWall }
	object{ HouseRearWall }
	object{ HouseFrontWall }
	object{ HouseFondations }
	object{ DoorObject }
	object{ RoofObject }
}
